@@ -1,4 +1,4 @@
 from .dist_utils import *  # noqa: F401, F403
 from .evaluation import *  # noqa: F401, F403
-from .fp16 import *  # noqa: F401, F403
+from .lr import *  # noqa: F401, F403
 from .optimizer import *  # noqa: F401, F403
