@@ -7,7 +7,9 @@ from .formating import (Collect, FormatShape, ImageToTensor, ToDataContainer,
 from .loading import (DecordDecode, DecordInit, DenseSampleFrames,
                       FrameSelector, GenerateLocalizationLabels,
                       LoadLocalizationFeature, LoadProposals, OpenCVDecode,
-                      OpenCVInit, PyAVDecode, PyAVInit, SampleFrames)
+                      OpenCVInit, PyAVDecode, PyAVInit, RawFrameDecode,
+                      SampleFrames, SampleProposalFrames,
+                      UntrimmedSampleFrames)
 
 __all__ = [
     'SampleFrames', 'PyAVDecode', 'DecordDecode', 'DenseSampleFrames',
@@ -16,5 +18,6 @@ __all__ = [
     'ThreeCrop', 'CenterCrop', 'TenCrop', 'ImageToTensor', 'Transpose',
     'Collect', 'FormatShape', 'Compose', 'ToTensor', 'ToDataContainer',
     'GenerateLocalizationLabels', 'LoadLocalizationFeature', 'LoadProposals',
-    'DecordInit', 'OpenCVInit', 'PyAVInit'
+    'DecordInit', 'OpenCVInit', 'PyAVInit', 'SampleProposalFrames',
+    'UntrimmedSampleFrames', 'RawFrameDecode'
 ]
