@@ -1,12 +1,13 @@
-from .backbones import (ResNet, ResNet2Plus1d, ResNet3d, ResNet3dSlowFast,
-                        ResNet3dSlowOnly, ResNetTSM)
+from .backbones import (ResNet, ResNet2Plus1d, ResNet3d, ResNet3dCSN,
+                        ResNet3dSlowFast, ResNet3dSlowOnly, ResNetTIN,
+                        ResNetTSM)
 from .builder import (build_backbone, build_head, build_localizer, build_model,
                       build_recognizer)
 from .common import Conv2plus1d
 from .heads import BaseHead, I3DHead, SlowFastHead, TSMHead, TSNHead
 from .localizers import BMN, PEM, TEM
 from .losses import (BCELossWithLogits, BinaryLogisticRegressionLoss, BMNLoss,
-                     CrossEntropyLoss, NLLLoss)
+                     CrossEntropyLoss, NLLLoss, OHEMHingeLoss, SSNLoss)
 from .recognizers import BaseRecognizer, recognizer2d, recognizer3d
 from .registry import BACKBONES, HEADS, LOCALIZERS, LOSSES, RECOGNIZERS
 
@@ -17,5 +18,6 @@ __all__ = [
     'BaseRecognizer', 'LOSSES', 'CrossEntropyLoss', 'NLLLoss', 'ResNetTSM',
     'ResNet3dSlowFast', 'SlowFastHead', 'Conv2plus1d', 'ResNet3dSlowOnly',
     'BCELossWithLogits', 'LOCALIZERS', 'build_localizer', 'PEM', 'TEM',
-    'BinaryLogisticRegressionLoss', 'BMN', 'BMNLoss', 'build_model'
+    'BinaryLogisticRegressionLoss', 'BMN', 'BMNLoss', 'build_model',
+    'OHEMHingeLoss', 'SSNLoss', 'ResNet3dCSN', 'ResNetTIN'
 ]
