@@ -3,8 +3,10 @@ from .binary_logistic_regression_loss import BinaryLogisticRegressionLoss
 from .bmn_loss import BMNLoss
 from .cross_entropy_loss import BCELossWithLogits, CrossEntropyLoss
 from .nll_loss import NLLLoss
+from .ohem_hinge_loss import OHEMHingeLoss
+from .ssn_loss import SSNLoss
 
 __all__ = [
     'BaseWeightedLoss', 'CrossEntropyLoss', 'NLLLoss', 'BCELossWithLogits',
-    'BinaryLogisticRegressionLoss', 'BMNLoss'
+    'BinaryLogisticRegressionLoss', 'BMNLoss', 'OHEMHingeLoss', 'SSNLoss'
 ]
